searchData={"items":[{"type":"module","title":"Qry","doc":"","ref":"Qry.html"},{"type":"behaviour","title":"Qry.Repo","doc":"Defines a repository.","ref":"Qry.Repo.html"},{"type":"callback","title":"Qry.Repo.fetch/3","doc":"","ref":"Qry.Repo.html#c:fetch/3"},{"type":"callback","title":"Qry.Repo.fetch/4","doc":"","ref":"Qry.Repo.html#c:fetch/4"},{"type":"type","title":"Qry.Repo.args/0","doc":"","ref":"Qry.Repo.html#t:args/0"},{"type":"type","title":"Qry.Repo.context/0","doc":"","ref":"Qry.Repo.html#t:context/0"},{"type":"type","title":"Qry.Repo.fetch_result/0","doc":"","ref":"Qry.Repo.html#t:fetch_result/0"},{"type":"type","title":"Qry.Repo.field/0","doc":"","ref":"Qry.Repo.html#t:field/0"},{"type":"type","title":"Qry.Repo.parent/0","doc":"","ref":"Qry.Repo.html#t:parent/0"},{"type":"extras","title":"Qry","doc":"# Qry\n\nQuery your domain.\n\n    [\n      project: [\n        :name,\n        author: [\n          :first_name,\n          :last_name,\n          links: [:name, :url]\n        ]\n      ]\n    ]\n    |> MyRepo.query()\n\n    {\n      :ok,\n      %{\n        project: %{\n          name: \"Qry\",\n          author: %{\n            first_name: \"Austin\",\n            last_name: \"Schneider\",\n            links: [\n              %{name: \"GitHub\", url: \"https://github.com/austinthecoder\"},\n              %{name: \"X\", url: \"https://twitter.com/austinthecoder\"}\n            ]\n          }\n        }\n      }\n    }","ref":"readme.html"},{"type":"extras","title":"Setup - Qry","doc":"defmodule MyRepo do\n      use Qry.Repo\n\n      @impl true\n      def fetch(field, args, context) do\n        ...\n      end\n\n      @impl true\n      def fetch(parent, field, args, context) do\n        ...\n      end\n    end\n\nThe repo must define two callbacks: `fetch/3` and `fetch/4`.\n\n`fetch/3` – Returns the field data.\n\n`fetch/4` – If the parent is a list, returns the a map of the field data keyed by each parent item. Otherwise returns the field data.","ref":"readme.html#setup"},{"type":"extras","title":"Basic Example - Qry","doc":"defmodule Project do\n      defstruct [:id, :name]\n    end\n\n    defmodule Repo do\n      use Qry.Repo\n\n      @project %Project{id: 1, name: \"Qry\"}\n\n      @impl true\n      def fetch(:project, _args, _context) do\n        {:ok, @project}\n      end\n    end\n\n    iex> MyRepo.query(project: [:name])\n    {:ok, %{project: %{name: \"Qry\"}}}\n\nLet's add an `:authors` association to `Project`:\n\n    defmodule Person do\n      defstruct [:id, :first_name, :last_name, :project_id]\n    end\n\n    defmodule Repo do\n      ...\n      @authors [\n        %Person{id: 1, first_name: \"Austin\", last_name: \"Schneider\", project_id: 1},\n        %Person{id: 2, first_name: \"John\", last_name: \"Smith\", project_id: 2},\n        %Person{id: 3, first_name: \"Sally\", last_name: \"Sue\", project_id: 1}\n      ]\n\n      def fetch(%Project{} = project, :authors, _args, _context) do\n        authors =\n          @authors\n          |> Enum.filter(fn author ->\n            author.project_id == project.id\n          end)\n\n        {:ok, authors}\n      end\n    end\n\n    iex> MyRepo.query(\n      project: [\n        :name,\n        authors: [:first_name, :last_name]\n      ]\n    )\n    {\n      :ok,\n      %{\n        project: %{\n          name: \"Qry\",\n          authors: [\n            %{first_name: \"Austin\", last_name: \"Schneider\"},\n            %{first_name: \"Sally\", last_name: \"Sue\"}\n          ]\n        }\n      }\n    }\n\nNow let's add a `:links` association to `Person`:\n\n    defmodule Link do\n      defstruct [:id, :name, :url, :person_id]\n    end\n\n    defmodule Repo do\n      ...\n      @links [\n        %Link{\n          id: 1,\n          name: \"GitHub\",\n          url: \"https://github.com/austinthecoder\",\n          person_id: 1\n        },\n        %Link{\n          id: 2,\n          name: \"X\",\n          url: \"https://twitter.com/austinthecoder\",\n          person_id: 1\n        },\n        %Link{\n          id: 3,\n          name: \"Example\",\n          url: \"https://example.com/john\",\n          person_id: 2\n        },\n        %Link{\n          id: 4,\n          name: \"Website\",\n          url: \"https://example.com/sally\",\n          person_id: 3\n        },\n      ]\n\n      def fetch([%Person{} | _] = people, :links, _args, _context) do\n        links =\n          people\n          |> Enum.reduce(%{}, fn person, acc ->\n            links = @links |> Enum.filter(fn link -> link.person_id == person.id end)\n            Map.put(acc, person, links)\n          end)\n\n        {:ok, links}\n      end\n    end\n\n    iex> MyRepo.query(\n      project: [\n        :name,\n        authors: [\n          :first_name,\n          :last_name,\n          links: [:name, :url]\n        ]\n      ]\n    )\n    {\n      :ok,\n      %{\n        project: %{\n          name: \"Qry\",\n          authors: [\n            %{\n              first_name: \"Austin\",\n              last_name: \"Schneider\",\n              links: [\n                %{name: \"GitHub\", url: \"https://github.com/austinthecoder\"},\n                %{name: \"X\", url: \"https://twitter.com/austinthecoder\"}\n              ]\n            },\n            %{\n              first_name: \"Sally\",\n              last_name: \"Sue\",\n              links: [\n                %{name: \"Website\", url: \"https://example.com/sally\"}\n              ]\n            }\n          ]\n        }\n      }\n    }","ref":"readme.html#basic-example"},{"type":"extras","title":"Documents - Qry","doc":"A doc consists of a field (atom), args (map), and subdocs (list). A doc can be expressed as an atom, a two-element tuple, or a three-element tuple.\n\nA atom can be used for a field with no args or subdocs:\n\n    MyRepo.query(:project)\n\nIf there are args or subdocs, a two-element tuple is used:\n\n    # args only\n    MyRepo.query({:project, %{id: \"p1\"}})\n\n    # subdocs only\n    MyRepo.query({:project, [:name]})\n\nIf there are both args and subdocs, a three-element tuple is used:\n\n    MyRepo.query({:project, %{id: \"p1\"}, [:name]})\n\nUse a list for multiple docs:\n\n    MyRepo.query([:project, {:users, [:name]}])\n\nNote: For two-element tuples, Elixir affords us the keyword list syntax:\n\n    MyRepo.query(project: [name], users: [:name])","ref":"readme.html#documents"},{"type":"extras","title":"Arguments - Qry","doc":"A doc can contain args (see above). They are given to `fetch/3` as the second argument:\n\n    MyRepo.query(project: %{foo: \"bar\"})\n\n    def fetch(:project, args, _context) do\n      # args are `%{foo: \"bar\"}`\n      ...\n    end\n\nAnd are given to `fetch/4` as the third argument:\n\n    MyRepo.query(project: [authors: %{foo: \"bar\"}])\n\n    def fetch(%Project{}, :authors, args, _context) do\n      # args are `%{foo: \"bar\"}`\n      ...\n    end","ref":"readme.html#arguments"},{"type":"extras","title":"Context - Qry","doc":"You can provide a context (map) as the second argument to `MyRepo.query/2`. It defaults to an empty map. It is given to `fetch/3` and `fetch/4` as the last argument.\n\n    MyRepo.query(:project, %{user_id: 123})\n    MyRepo.query(project: [:authors], %{user_id: 123})\n\n    def fetch(:project, _args, context) do\n      # context is `%{user_id: 123}`\n      ...\n    end\n\n    def fetch(%Project{}, :authors, _args, context) do\n      # context is `%{user_id: 123}`\n      ...\n    end","ref":"readme.html#context"}],"content_type":"text/markdown"}